/*istanbul ignore next*/"use strict";

exports.__esModule = true;

exports.default = function ( /*istanbul ignore next*/_ref) {
  /*istanbul ignore next*/var t = _ref.types;

  function wrapInFlowComment(path, parent) {
    path.addComment("trailing", generateComment(path, parent));
    path.replaceWith(t.noop());
  }

  function generateComment(path, parent) {
    var comment = path.getSource().replace(/\*-\//g, "*-ESCAPED/").replace(/\*\//g, "*-/");
    if (parent && parent.optional) comment = "?" + comment;
    if (comment[0] !== ":") comment = ":: " + comment;
    return comment;
  }

  return {
    inherits: require("babel-plugin-syntax-flow"),

    visitor: { /*istanbul ignore next*/
      TypeCastExpression: function TypeCastExpression(path) {
        /*istanbul ignore next*/var node = path.node;

        path.get("expression").addComment("trailing", generateComment(path.get("typeAnnotation")));
        path.replaceWith(t.parenthesizedExpression(node.expression));
      },
      /*istanbul ignore next*/

      // support function a(b?) {}
      Identifier: function Identifier(path) {
        /*istanbul ignore next*/var node = path.node;

        if (!node.optional || node.typeAnnotation) {
          return;
        }
        path.addComment("trailing", ":: ?");
      },


      // strip optional property from function params - facebook/fbjs#17
      Function: { /*istanbul ignore next*/
        exit: function exit(_ref2) {
          /*istanbul ignore next*/var node = _ref2.node;

          node.params.forEach(function (param) /*istanbul ignore next*/{
            return param.optional = false;
          });
        }
      },

      /*istanbul ignore next*/ // support `export type a = {}` - #8 Error: You passed path.replaceWith() a falsy node
      "ExportNamedDeclaration|Flow": function ExportNamedDeclarationFlow(path) {
        /*istanbul ignore next*/var node = path.node;
        /*istanbul ignore next*/var parent = path.parent;

        if (t.isExportNamedDeclaration(node) && !t.isFlow(node.declaration)) {
          return;
        }
        wrapInFlowComment(path, parent);
      },
      /*istanbul ignore next*/

      // support `import type A` and `import typeof A` #10
      ImportDeclaration: function ImportDeclaration(path) {
        /*istanbul ignore next*/var node = path.node;
        /*istanbul ignore next*/var parent = path.parent;

        if (t.isImportDeclaration(node) && node.importKind !== "type" && node.importKind !== "typeof") {
          return;
        }
        wrapInFlowComment(path, parent);
      }
    }
  };
};

/*istanbul ignore next*/module.exports = exports["default"];